/************************************************************
 * November 20, 1996 - Steve Sprang (sprang@andrew.cmu.edu)
 * 		- Modified VideoView class from Be to work with my
 *		  interface
 ***********************************************************/
 
#include "VideoView.h"
#include "VideoWindow.h"
#include "qcam_driver.h"

#define bmsgVideoFrame 'vidf'

VideoView::VideoView(BRect r, ulong size, int bpp)
	: BView(r,"VideoView",B_FOLLOW_ALL,B_WILL_DRAW)
{
	m_qcamDevice = open("/dev/quickcam",O_RDONLY|O_BINARY);
	qcam qc;
	
	// need this later
	m_sem = create_sem(1, "");
	
	ioctl(m_qcamDevice,QC_GET,&qc);
	
	qc.brightness = 168;
	qc.contrast = 124;
	qc.whitebalance = 210; // your mileage may vary
	
	CreateBitmaps(size, qc);
	
	switch (bpp) {
		case 4:
			qc.bpp = QC_4BIT_PIXEL;
			break;
		case 6:
			qc.bpp = QC_6BIT_PIXEL;
			break;
	};
	
	ioctl(m_qcamDevice,QC_SET,&qc);
}

void VideoView::CreateBitmaps(ulong size, qcam &qc)
{
	switch (size) {
		case QCAM_80x60:
			qc.factor = QC_FACTOR_41;
			m_numPix = 80*60;
			m_bitmap1 = new BBitmap(BRect(0,0,79,59),B_COLOR_8_BIT,FALSE);
			m_bitmap2 = new BBitmap(BRect(0,0,79,59),B_COLOR_8_BIT,FALSE);
			break;
		case QCAM_160x120:
			qc.factor = QC_FACTOR_21;
			m_numPix = 160*120;
			m_bitmap1 = new BBitmap(BRect(0,0,159,119),B_COLOR_8_BIT,FALSE);
			m_bitmap2 = new BBitmap(BRect(0,0,159,119),B_COLOR_8_BIT,FALSE);
			break;
		case QCAM_320x240:
			qc.factor = QC_FACTOR_11;
			m_numPix = 320*240;
			m_bitmap1 = new BBitmap(BRect(0,0,319,239),B_COLOR_8_BIT,FALSE);
			m_bitmap2 = new BBitmap(BRect(0,0,319,239),B_COLOR_8_BIT,FALSE);
			break;
	};

	m_bitmap = m_bitmap1;
	m_nextBitmap = m_bitmap2;
}

VideoView::~VideoView()
{
	close(m_qcamDevice);
	delete m_bitmap;
	delete m_nextBitmap;
	delete_sem(m_sem);
}

void VideoView::AttachedToWindow()
{
	inherited::AttachedToWindow();
	Window()->PostMessage(new BMessage(bmsgVideoFrame),this);
}

void VideoView::DoFrame()
{	
	// because we may be resizing the bitmaps at any time
	// protect this with a semaphore
	acquire_sem(m_sem);
	
	read(m_qcamDevice,m_nextBitmap->Bits(),m_numPix);
	BBitmap *bm = m_bitmap;
	m_bitmap = m_nextBitmap;
	m_nextBitmap = bm;
	// no longer async, so that we properly hang on to the semaphore
	DrawBitmap(m_bitmap,m_bitmap->Bounds(),Bounds());
	
	release_sem(m_sem);
}

void VideoView::ResizeBitmaps(ulong msize)
{
	qcam qc;
	ulong size;
	
	// because we are always trying to draw frames
	// protect this with a semaphore
	acquire_sem(m_sem);
	
	delete m_bitmap1;
	delete m_bitmap2;
	
	ioctl(m_qcamDevice,QC_GET,&qc);
	
	switch(msize) {
		case mSMALL:
			size = QCAM_80x60;
			break;
		case mMEDIUM:
			size = QCAM_160x120;
			break;
		case mLARGE:
			size = QCAM_320x240;
			break;
	}
	
	CreateBitmaps(size, qc);
	
	Window()->ResizeTo(m_bitmap->Bounds().Width(),
		m_bitmap->Bounds().Height() + kMenuSize + kToolSize);
	
	ioctl(m_qcamDevice,QC_SET,&qc);
	
	release_sem(m_sem);
}

void VideoView::AdjustBitDepth(ulong bpp)
{
	qcam qc;
	ioctl(m_qcamDevice,QC_GET,&qc);
	
	switch (bpp) {
		case m4BITS:
			qc.bpp = QC_4BIT_PIXEL;
			break;
		case m6BITS:
			qc.bpp = QC_6BIT_PIXEL;
			break;
	}
	ioctl(m_qcamDevice,QC_SET,&qc);
}

void VideoView::UpdateBrightness(ulong val)
{
	qcam qc;
	
	ioctl(m_qcamDevice,QC_GET,&qc);
	qc.brightness = (int) val;
	ioctl(m_qcamDevice,QC_SET,&qc);
}

// secret feature! Shh!
void VideoView::KeyDown(ulong aChar)
{
	qcam qc;
	
	if(aChar == '.' || aChar == ',') {
		ioctl(m_qcamDevice,QC_GET,&qc);
		switch(aChar) {
			case '.':
				qc.brightness++;
				break;
			case ',':
				qc.brightness--;
				break;
		}
		ioctl(m_qcamDevice,QC_SET,&qc);
	}
}

void VideoView::MessageReceived(BMessage *msg)
{
	switch (msg->what) {
		case bmsgVideoFrame:
			DoFrame();
			Window()->PostMessage(new BMessage(bmsgVideoFrame),this);
			break;
		default:
			inherited::MessageReceived(msg);
	};
};

void VideoView::Draw(BRect updateRect)
{
	DrawBitmap(m_bitmap,m_bitmap->Bounds(),Bounds());
};
