/************************************************************
 * November 20, 1996 - Steve Sprang (sprang@andrew.cmu.edu)
 * 		- Created the VideoWindow and BrightnessScroll 
 *		  classes to interface with the basic code from Be.
 ***********************************************************/
 
#include "VideoView.h"
#include "VideoWindow.h"

/*******************
 * QuitFilter 
 ******************/
 
class QuitFilter : public BMessageFilter
{
public:
						QuitFilter()
							: BMessageFilter(	B_ANY_DELIVERY,
												B_ANY_SOURCE,
												B_QUIT_REQUESTED) {};
virtual	filter_result 	Filter(BMessage *msg, BHandler **target);
};

filter_result QuitFilter::Filter(BMessage *msg, BHandler **target)
{
	be_app->PostMessage(B_QUIT_REQUESTED);
	return B_DISPATCH_MESSAGE;
};


/**********************
 * BrightnessScroll 
 *********************/
 
class BrightnessScroll : public BScrollBar
{
public:
			BrightnessScroll(BRect r, const char *name, VideoView *vv, 
				VideoWindow *w, long min, long max, orientation posture)
					: BScrollBar(r, name, NULL, min, max, posture) 
			{
				this->vv = vv;
				this->w = w;
			};
			
virtual void		 	ValueChanged(long val);
	
private:
		VideoView 	*vv;
		VideoWindow *w;
};

void BrightnessScroll::ValueChanged(long val)
{
	// adjust the image
	vv->UpdateBrightness(val);
	// update the brightness info
	w->UpdateInfo(val);
};

/*******************
 * VideoWindow 
 ******************/
 
VideoWindow::VideoWindow(BRect bounds, const char *name)
	: BWindow(bounds, name, B_TITLED_WINDOW, NULL)
{
	BView	*bg;
	
	Lock();
	
	// create a background view
	bounds.OffsetTo(B_ORIGIN);
	bg = new BView(bounds, "", B_FOLLOW_ALL, B_WILL_DRAW);
	bg->SetViewColor(210,210,210);
	
	// create the menus
	CreateMyMenus(bounds);
	
	// add the video view
	bounds.top = kMenuSize;
	bounds.bottom -= kToolSize;	// make room for controls
	vv = new VideoView(bounds,QCAM_160x120,6);
	
	// create an ad-hoc slider to control brightness
	BRect frame;
	frame.Set(10, 150, 120, 164);
	scroll = new BrightnessScroll(frame, "", vv, this,
 0 , 255 , B_HORIZONTAL);
	
	// create a little text field to indicate the value of brightness
	frame.Set(130, 150, 150, 164);
	info = new BStringView(frame, "", "168", B_FOLLOW_BOTTOM | B_FOLLOW_RIGHT);
	info->SetViewColor(210,210,210);
	
	// add the views
	bg->AddChild(menu);
	bg->AddChild(vv);
	bg->AddChild(scroll);
	bg->AddChild(info);
	AddChild(bg);
	
	// housekeeping
	vv->MakeFocus();
	scroll->SetValue(168);
	scroll->SetProportion(0.1);
	AddCommonFilter(new QuitFilter());
	
	Unlock();
}

void VideoWindow::UpdateInfo(long val)
{
	char value[16];
	sprintf(value, "%d", (int) val);
	
	info->SetText(value);
}

// should be self-explanatory
void VideoWindow::CreateMyMenus(BRect r)
{
	BMenu 		*size, *depth;
	BMenuItem	*newItem;
	
	r.bottom = kMenuSize;
	menu = new BMenuBar(r, "");
	
	size = new BMenu("Size", B_ITEMS_IN_COLUMN);
	depth = new BMenu("Depth", B_ITEMS_IN_COLUMN);
	
	size->SetRadioMode(TRUE);
	depth->SetRadioMode(TRUE);
	
	newItem = new BMenuItem("80 x 60", new BMessage(mSMALL), NULL, NULL);
	size->AddItem(newItem);
	newItem = new BMenuItem("160 x 120", new BMessage(mMEDIUM), NULL, NULL);
	size->AddItem(newItem);
	newItem->SetMarked(TRUE);
	newItem = new BMenuItem("320 x 240", new BMessage(mLARGE), NULL, NULL);
	size->AddItem(newItem);
	
	newItem = new BMenuItem("4 bits", new BMessage(m4BITS), NULL, NULL);
	depth->AddItem(newItem);
	newItem = new BMenuItem("6 bits", new BMessage(m6BITS), NULL, NULL);
	depth->AddItem(newItem);
	newItem->SetMarked(TRUE);
	
	menu->AddItem(size);
	menu->AddItem(depth);
}


void VideoWindow::MessageReceived(BMessage *msg)
{
	switch(msg->what) {
		case mSMALL:
		case mMEDIUM:
		case mLARGE:
			vv->ResizeBitmaps(msg->what);
			break;
		case m4BITS:
		case m6BITS:
			vv->AdjustBitDepth(msg->what);
			break;
	}
}


