/*
 * Thanks to Scot Laird, Russell Nelson, and others..
 *
 * Copyright (C) 1996 by Thomas Davis
 *
 * Copyright (C) 1996 by Scott Laird
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:

 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL SCOTT LAIRD BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 */


#ifndef _QUICKCAM_H
#define _QUICKCAM_H 1

#ifndef _DRIVERS_H
#include <Drivers.h>
#endif

#define FRAME_BUFFER_SIZE	336*243

/*
 * these are private to the driver..
 */

#define QC_IO_BASE				0x378
#define QC_DATA_REGISTER		0
#define QC_STATUS_REGISTER		1
#define QC_CONTROL_REGISTER		2

#define QC_BI_TEST_BYTE			0x75

/* standard lpt control bits */
#define LP_INTEN	0x10
#define LP_SELECP	0x08  /* inverted output, active low */
#define LP_INITP	0x04  /* unchanged output, active low */
#define LP_AUTOLF	0x02  /* inverted output, active low */
#define LP_STROBE	0x01  /* inverted output, active low */

/*
	Transfer Mode
*/
enum {
	QC_NYBBLE_TRANSFER = 0,
	QC_BYTE_TRANSFER = 1
};

/*
	Bits Per Pixel
*/

enum {
	QC_4BIT_PIXEL	= 0,
	QC_6BIT_PIXEL	= 2
};

/*
	Compression factor
*/

enum {
	QC_FACTOR_11 = 0,
	QC_FACTOR_21 = 4,
	QC_FACTOR_41 = 8
};

enum {
	QC_SENDVIDEO	= 7,
	QC_SETEXPOSURE	= 11,
	QC_SETTOP 		= 13,
	QC_SETLEFT		= 15,
	QC_SETNUMV		= 17,
	QC_SETNUMH		= 19,
	QC_SENDVERSION	= 23,
	QC_SETCONTRAST	= 25,
	QC_AUTOCALIBRATE = 27,
	QC_BYTEPORTECHO	= 29,
	QC_SETBLACKLEVEL = 31,
	QC_GETBLACKLEVEL = 33
};

#define CamRdy1	8

/*
	Control Register bit twiddling
*/

#define Reset_N_HIGH	4
#define Reset_N_OFF	0
#define PCAck_LOW	8
#define PCAck_HIGH	0
enum {
	PCAckRaise = 0x07,
	PCAckLower = 0x0f
};

/*
 * these are the ioctls..
 */

enum {
	QC_SET		= B_DEVICE_OP_CODES_END + 1,
	QC_GET
};


/*
	This struct qcam is used by client applications that need
	to send or receive information through the ioctl call.
	It is enough information to control the camera.
*/

struct qcam {
	int bpp;
	int mode;
	int	factor;
	unsigned char brightness;
	unsigned char whitebalance;
	unsigned char contrast;
	unsigned char status;
};


#endif
